--
-- Adobe Photoshop Lightroom export plugin for Flickr, Flickr API routines
-- Copyright 2008 Jeffrey Friedl
--

API = { }
local LrDialogs   = import 'LrDialogs'
local LrView      = import 'LrView'
local LrHttp      = require 'LrHttpDebug' { logging = false, trapping = false}
local LrFileUtils = import 'LrFileUtils'
local LrPathUtils = import 'LrPathUtils'

API.LrHttp = LrHttp

local util        = require 'util'
local key         = require 'data'
local tableutil   = require 'table-util'

API.auto_reauthenticate = true

local FLICKR_INVALID_AUTH_TOKEN = '98'

local function grokError(xml)

   if not xml or xml == '' then
      API.errorCode = '999'
      API.errorMessage = LOC("$$$/372=empty response")
   elseif xml == 'TRAPPED ERROR' then
      API.errorCode = '999'
      API.errorMessage = LOC("$$$/373=trapped error")
   else
      local code = xml:match('code="(.-)"')
      local msg  = xml:match('msg="(.-)"')

      API.errorCode = code
      API.errorMessage = msg
   end

   return xml
end


function API.call(args)

   local ORIG_ARGS = tableutil.simplecopy(args)

   assert(args.state)
   local state = args.state
   args.state = nil

   --
   -- If auth_token is 'true' (as opposed to any other specific value), we
   -- must replace it with the auth_token from the state, or get one if it's
   -- not already there
   --

   if args.auth_token == true then
      if not state.flickr_auth_token then
         if not API.auto_reauthenticate then
            return false
         end
         if not API.getAuthToken(state) then
            return false
         end
      end
      args.auth_token = state.flickr_auth_token
   end

   --
   -- We always want our API key in the calls.
   --
   args.api_key = API.key(state)

   --
   -- Pseudo-argument "__return_url" -- if true, just return the URL we create for
   -- the call, rather than actually fetching the url and returning the response.
   --
   local return_url

   if args.__return_url then
      return_url = true
      args.__return_url = nil
   end

   local empty_okay
   if args.__empty_okay then
      empty_okay = true
      args.__empty_okay = nil
   end

   --
   -- Pseudo-argument "__url" -- if given, use it as the base of the url to fetch.
   -- Otherwise, we use the standard Flickr API url.
   --
   local url = 'http://api.flickr.com/services/rest/'  -- default url to use

   if args.__url then
      url = args.__url -- overwrite default with one we were given
      args.__url = nil
   end

   --
   -- Sign it, unless we're told not to
   --
   local sign = true

   if args.__sign ~= nil then
      lign = args.__sign
      args.__sign = nil
   end


   if sign then
      API.sign_table(state, args)
   end

   --
   -- Append the key/value pairs for the call to the base url
   --
   url = url .. '?'

   for k,v in pairs(args) do
      url = url .. util.urlencode(k) .. '=' .. util.urlencode(v) .. '&'
   end

   if return_url then
      return url
   end

   local raw_result = LrHttp.get(url)

   if empty_okay and raw_result and raw_result == '' then
      return ''
   end

   local result = grokError(raw_result)

   if API.errorCode == FLICKR_INVALID_AUTH_TOKEN then
      API.logout(state)
      ORIG_ARGS.auth_token = true
      result = API.call(ORIG_ARGS)
   end

   return result
   
end



local function grok_token_response(state, ReplyXML)


   local token = ReplyXML:match('<token>(.-)</token>')

   if token then

      state.flickr_auth_token = token

      local nsid = ReplyXML:match('nsid="(.-)"')

      local InfoXML = API.call {
         state = state,
         method = 'flickr.people.getInfo',
         user_id = nsid,
         auth_token = true -- 'true' here means that the auth_token should be auto-filled-in
      }

      if not InfoXML then
         return false
      end

      state.flickr_home_url = InfoXML:match('<photosurl>(.-)</photosurl>')
      state.flickr_name = ReplyXML:match('fullname="(.-)"')
      if ReplyXML:match('ispro="1"') then
         state.flickr_is_pro = true
      else
         state.flickr_is_pro = false
      end

      if not state.flickr_name or #state.flickr_name == 0 then 
         state.flickr_name = ReplyXML:match('username="(.-)"')
         if not state.flickr_name or #state.flickr_name == 0 then 
            state.flickr_name = LOC("$$$/374=(name not known)")
         end
      end


      return true
   else
      return false
   end

end

function API.logout(state)

   state.flickr_auth_token          = nil
   state.flickr_name                = nil
   state.flickr_home_url            = nil
   state.flickr_bandwidth_max       = nil
   state.flickr_filesize_max        = nil
   state.flickr_bandwidth_used      = nil
   state.flickr_bandwidth_remain    = nil
   state.flickr_can_create_new_sets = false
   state.flickr_is_pro              = false
   state.flickr_photoset2name       = {}

end

function API.getAuthToken(state)

   local v = LrView.osFactory()

   --
   -- Be sure to flush everything in the state that we think we know about the user
   --
   API.logout(state)

   state._account_synopsis = LOC("$$$/375=awaiting dialog")

   --
   -- Let the user know that they're being shoved over to their browser for authentication at Flickr....
   --
   local UserReply = LrDialogs.presentModalDialog {
      title = LOC("$$$/376=Jeffrey's Lightroom Export-to-Flickr Plugin"),
      resizable = false,
      contents = v:row {
         v:static_text {
            height_in_lines = 5,
            title = LOC("$$$/377=You must visit Flickr.com to allow this plugin access to your Flickr account.^n^n^n" ..
                        "Click [OK] to load Flickr.com's authentication page into your web browser.^n" ..
                        "After you log in and authenticate there, return to Lightroom to complete the process.") }
      }
   }

   if UserReply ~= 'ok' then
      state._account_synopsis = LOC("$$$/378=Not authenticated to Flickr")
      return false
   end

   state._account_synopsis = LOC("$$$/379=Authenticating at Flickr.com^.")

   --
   -- Shove them over to Flickr for authentication
   --
   local frobXml = API.call { state = state,
                              method = 'flickr.auth.getFrob' }

   if not frobXml then
      return false
   end

   local frob = frobXml:match('<frob>(.-)</frob>')
   local url = API.call { state = state,
                          __return_url = true,
                          __url = 'http://flickr.com/services/auth/',
                          perms = 'delete',
                          frob = frob }

   if not url then
      return false
   end

   LrHttp.openUrlInBrowser(url)

   --
   -- Place a dialog for the user to indicate that they've authenticated.
   --
   UserReply = LrDialogs.presentModalDialog {
      title = LOC("$$$/376=Jeffrey's Lightroom Export-to-Flickr Plugin"),
      actionVerb = LOC("$$$/380=I've authenticated at Flickr.com"),
      contents = v:row {
         v:static_text { title = LOC("$$$/381=After authenticating with the web browser, continue here^.") }
      }
   }

   if UserReply ~= 'ok' then
      state._account_synopsis = LOC("$$$/378=Not authenticated to Flickr")
      return false
   end

   local ReplyXML = API.call { state  = state,
                               method = 'flickr.auth.getToken',
                               frob   = frob }

   if not ReplyXML then
      return false
   end

   return grok_token_response(state, ReplyXML)
end

function API.tokenIsValid(state)

   if not state.flickr_auth_token then
      return false
   end

   state._account_synopsis = LOC("$$$/382=Checking authentication at Flickr.com^.")

   local ReplyXML = API.call { state = state,
                               method = 'flickr.auth.checkToken',
                               auth_token = state.flickr_auth_token }


   if not ReplyXML or not grok_token_response(state, ReplyXML) then
      -- not valid, so consider us logged out
      API.logout(state)
      return false
   else
      return true
   end

end


function API.uploadPhoto(state, filepath, args)

   if not state.flickr_auth_token and  not API.getAuthToken(state) then
      return false
   end


   local attribs = LrFileUtils.fileAttributes(filepath)
   if attribs and state.flickr_filesize_max and attribs.fileSize > state.flickr_filesize_max then
      LrDialogs.message(LOC("$$$/439=Exported Size of ^[^1^] Exceeds Your ^2 Account Limits", LrPathUtils.leafName(filepath), "Flickr"),
                        LOC("$$$/440=The exported image size of ^1 bytes exceeds your account image-size limit of ^2 bytes.^nThe export has been canceled.",
                            util.C(attribs.fileSize), util.C(state.flickr_filesize_max)), "error");
      API.errorCode    = 9998 -- magic 
      return ""
   end

   args.api_key    = API.key(state)
   args.auth_token = state.flickr_auth_token

   local url
   if args.photo_id then
      url = 'http://api.flickr.com/services/replace/'
   else
      url = 'http://api.flickr.com/services/upload/'
   end

   API.sign_table(state, args)

   local parts = {}

   for k,v in pairs(args) do
      table.insert(parts, { name = k, value = v })
   end

   local nameonly = filepath:match('([^/\\]+)$')

   table.insert(parts, {
                   name = 'photo',
                   fileName = nameonly,
                   filePath = filepath,
                   contentTYpe = 'image/' .. string.lower(state.LR_format)
                })

   return grokError(LrHttp.postMultipart(url, parts))
end

function API.deletePhoto(state, photoId)

   assert(photoId)

   local result = API.call {
      state      = state,
      method     = 'flickr.photos.delete',
      auth_token = true, -- 'true' here means that the auth_token should be auto-filled-in
      photo_id   = photoId,
   }

end

function API.addToPhotoset(state, photoId, photosetId)

   local result = API.call { state = state,
                             method = 'flickr.photosets.addPhoto',
                             auth_token = true, 
                             photo_id = photoId,
                             photoset_id = photosetId }
end

local xmlHeader = '<?xml version="1.0" encoding="utf-8" ?>'

function API.default_photoset_list(can_add_new)

   -- if 'can_add_new' is nil, it's treated as 'true', so we don't allow adding only if we explicitly know it's false

   local default_photosets = {}

   table.insert(default_photosets, { title = "-- " .. LOC("$$$/383=don't add to a photoset") .. " --", value = ''   })

   if can_add_new ~= false then
      table.insert(default_photosets, { title = "-- " .. LOC("$$$/384=add to new photoset") .. " -- ", value = 'new' })
   end

   return default_photosets
end

function API.setPhotoDate(state, photoId, dateString)

   local result = API.call { state = state,
                             method = 'flickr.photos.setDates',
                             auth_token = true,
                             photo_id = photoId,
                             date_taken = dateString }

end


function API.updateBandwidthStats(state)

   local replyXML = API.call {
      state = state,
      method = 'flickr.people.getUploadStatus', -- http://flickr.com/services/api/flickr.people.getUploadStatus.html
      auth_token = true -- 'true' here means that the auth_token should be auto-filled-in
   }

   if not replyXML then
      return false
   end

   -- require 'jfriedlDebug.lua' Dump(replyXML);

   local bandwidth = replyXML:match('<bandwidth.-/>');
   local filesize  = replyXML:match('<filesize.-/>');

   state.flickr_bandwidth_max    = tonumber(bandwidth:match('maxkb="(.-)"'))
   state.flickr_bandwidth_used   = tonumber(bandwidth:match('usedkb="(.-)"'))
   state.flickr_bandwidth_remain = tonumber(bandwidth:match('remainingkb="(.-)"'))
   state.flickr_filesize_max     = tonumber(filesize:match('maxbytes="(.-)"'))

   if replyXML:match('ispro="1"') then
      state.flickr_is_pro = true
   else
      state.flickr_is_pro = false
   end

   if replyXML:match('remaining="0"') then
      state.flickr_can_create_new_sets = false
   else
      state.flickr_can_create_new_sets = true
   end

   return true
end

function API.sort_photoset_list(state)

   local tmp = state.flickr_photosets

   table.sort(tmp, function(a,b)
                      -- Items that begin with "--" always go at the end of the list
                      if a.title:match('^%-%-') then
                         if b.title:match('^%-%-') then
                            return string.lower(a.title) < string.lower(b.title)
                         else
                            return false
                         end
                      end

                      if b.title:match('^%-%-') then
                         return true
                      end

                      if state.flickr_photoset_sort == 'Title' then
                         return string.lower(a.title) < string.lower(b.title)
                      else
                         return a.value > b.value
                      end
                   end)

   --Dump({ key = state.flickr_photoset_sort, table = tmp }, "sorted")

   state.flickr_photosets = tmp -- so that LrBind notices the change
end



function API.getPhotosets(state)

   state._getting_photoset_list = true

   local result = API.call { state = state, method = 'flickr.photosets.getList', auth_token = true }

   if not result then
      state._getting_photoset_list = nil
      return false
   end

   if result:match('cancreate="1"') then
      state.flickr_can_create_new_sets = true
   else
      state.flickr_can_create_new_sets = false
   end

   local photosets = API.default_photoset_list(state.flickr_can_create_new_sets)
   interactive_destination.add_select_element(state, photosets)

   state.flickr_photoset2name = {}

   local count = 0
   for setXml in result:gmatch('<photoset.-</photoset>') do
      local id    = setXml:match('id="(%d+)"')
      local title = setXml:match('<title>(.-)</title>')
      title = title:gsub('&lt;',  '<')
      title = title:gsub('&gt;',  '>')
      title = title:gsub('&amp;', '&')

      table.insert(photosets, { title = title, value = id })
      state.flickr_photoset2name[id] = title
      count = count + 1
   end

   state.flickr_sortable_photosets = count  >= 2
   util.tidy_popupmenu_items(photosets)

   state.flickr_photosets = photosets

   if #photosets == 0 then
      state._have_some_photosets = false
      state.flickr_destination_photoset = ''
   else

      state._have_some_photosets = true

      -- set the destination photoset to the first one if the previously-chosen photoset
      -- is no longer in there.
      if not util.popupmenu_items_has_value(photosets, state.flickr_destination_photoset) then
         state.flickr_destination_photoset = photosets[1].value
      end
   end

   API.sort_photoset_list(state)
   state._getting_photoset_list = nil
   return true
end

function API.refreshUserData(state)
   return API.getPhotosets(state) and API.updateBandwidthStats(state)
end

function API.refreshUserDataWithoutAutoAuthentication(state)
   API.auto_reauthenticate = false
   API.refreshUserData(state)          
   API.auto_reauthenticate = true
end


function API.createPhotoset(state, title, description, photoId)

   local result = API.call { state            = state,
                             method           = 'flickr.photosets.create',
                             auth_token       = true,
                             title            = title,
                             description      = description,
                             primary_photo_id = photoId,
                          }

   if not result then
      return false
   else
      return result:match('photoset id="(%d+)"')
   end
end

return API

