--
-- Copyright 2008 Jeffrey Friedl
-- http://regex.info/blog/
--
--
-- This module's "run" function makes it easier for a plugin to execute
-- a function in an separate task
--
local LrFunctionContext = import 'LrFunctionContext'
local LrDialogs         = import 'LrDialogs'

local async_task = {}

--
-- Given the plugin's STATE and a LABEL describing what the given FUNC
-- does, calls FUNC(STATE, EXTRA_ARG) in an asynchronous task. The
-- EXTRA_CLEANUP function, if provided, is called when the task exits.
-- If the task exited normally, it's called as EXTRA_CLEANUP(STATE, true),
-- and if the task aborted for any reason, it's called as
-- EXTRA_CLEANUP(STATE, true)
--
function async_task.run(state, label, func, extra_arg, extra_cleanup)

   local ToRun = function(context)
                    state.__in_progress = true

                    LrDialogs.attachErrorDialogToFunctionContext(context)

                    --
                    -- Add a cleanup handler that re-enables the dialog's buttons, but also
                    -- issues a report if the function aborted early for some reason.
                    --
                    context:addCleanupHandler(function()
                                                 local normal = true

                                                 -- if the flag is still set, we aborted early
                                                 if state.__in_progress then
                                                    
                                                    normal = false
                                                    local message = label .. " " .. LOC("$$$/174=aborted")
                                                    if state.__in_progress ~= true then
                                                       message = message .. " (" .. tostring(state.__in_progress) .. ")"
                                                    end

                                                    LrDialogs.message(LOC("$$$/101=INTERNAL PLUGIN ERROR"), message, "critical")
                                                 end

                                                 -- do any extra cleanup requested
                                                 if extra_cleanup then
                                                    extra_cleanup(state, normal)
                                                 end

                                                 -- in any case, re-enable buttons
                                                 state._buttons_are_enabled = true
                                              end)

                    state._buttons_are_enabled = false

                    --
                    -- Within the function, we can set state.__in_progress
                    -- to information about what we're doing, and if we abort
                    -- while doing it, it'll be reported by the cleanup handler
                    --
                    func(state, extra_arg)


                    --
                    -- By setting this to nil, we're telling the cleanup handler that
                    -- we didn't abort, but rather executed through to completion.
                    -- We enter the cleanup handler when dropping off the end of this
                    -- function.
                    --
                    state.__in_progress = nil
                 end

   LrFunctionContext.postAsyncTaskWithContext(label, ToRun)
end

--
-- Return a table that exposes the public functions of this module
--
return async_task
