--
-- Copyright 2008 Jeffrey Friedl
-- http://regex.info/blog/
--

local LrView    = import 'LrView'
local LrBinding = import 'LrBinding'
local LrColor   = import 'LrColor'

local util    = require 'util'

local SAMPLE  = LrColor(.5,.5,1)
local OKAY    = LrColor(0,0,0)
local ERROR   = LrColor(1,0,0)


local extra_tags = {}

-- provider          e.g. "Flickr"
-- extra_tags_key    key in 'state' that holds the extra tags
-- default           'space' or 'comma'

local my


function extra_tags.check(state)

   local note
   local tags, okay = extra_tags.as_table(state)
   state._extra_tag_count = tostring(#tags)

   if okay == true then
      state._extra_tag_count_color = OKAY
      if #tags == 1 then
         note = LOC("$$$/278/new=1 ^2 [ ^1 ]", state[my.extra_tags_key], my.tag)
      elseif #tags > 1 then
         note = LOC("$$$/279/new=^1 ^3 [ ^2 ]", #tags, state[my.extra_tags_key], my.tags)
      end
   else
      state._extra_tag_count_color = ERROR
      note = LOC("$$$/280=EXTRA TAGS MALFORMED")
   end

   return note
end


function extra_tags.dialog_section(v, state, call_on_change)

   assert(my.extra_tags_key)

   function on_change()
      extra_tags.check(state)
      if call_on_change then
         call_on_change(state)
      end
   end

   state:addObserver(my.extra_tags_key,   on_change)
   state:addObserver('_extra_tags_style', on_change)

   return v:column {
      v:view {
         spacing = 0,
         margin = 0,
         place = 'horizontal',
         fill_horizontal = 1,

         v:static_text { title = my.header },
         v:static_text { title = " (" },
         v:static_text {
            title      = LrView.bind("_extra_tag_count"),
            text_color = LrView.bind("_extra_tag_count_color"),
         },
         v:static_text { title = ")" },

         v:edit_field {
            fill_horizontal = 1,
            immediate = true,
            value = LrView.bind(my.extra_tags_key),
         },
      },

      v:view {
         place = 'horizontal',
         spacing = 0,
         margin = 0,

         v:spacer {
            width = tonumber(util.WinMac(LOC("$$$/Indent1/Win=56"),
                                         LOC("$$$/Indent1/Mac=34"))),
         },

         v:radio_button {
            size = 'mini',
            title = " ",
            checked_value = 'comma',
            value = LrView.bind("_extra_tags_style"),
            tooltip = LOC("$$$/281/new=Select this to choose Lightroom-style (comma separated) ^1", my.tags),
         },

         v:radio_button {
            size = 'mini',
            title = " ",
            checked_value = 'space',
            value = LrView.bind("_extra_tags_style"),
            tooltip = LOC("$$$/282/new=Select this to choose ^1-style (space separated) ^2", my.provider, my.tags),
         },

         v:view {
            place = 'overlapping',
            spacing = 0,
            margin = 0,

            v:view {
               visible = LrBinding.keyEquals('_extra_tags_style', 'space'),
               place = 'horizontal',
               spacing = 0,
               margin = 0,

               v:static_text { title = "(" },

               v:static_text {
                  title = my.space_help,
                  font = "<system/small>"
               },

               v:static_text {
                  title = my.space_example,
                  font = "<system/small/bold>",
                  text_color = SAMPLE
               },

               v:static_text {
                  title = ")",
                  font = "<system/small>"
               }
            },
            v:view {
               visible = LrBinding.keyEquals('_extra_tags_style', 'comma'),
               place = 'horizontal',
               margin = 0,
               spacing = 0,

               v:static_text { title = "(" },

               v:static_text {
                  title = my.comma_help,
                  font = "<system/small>"
               },

               v:static_text {
                  title = my.comma_example,
                  font = "<system/small/bold>",
                  text_color = SAMPLE
               },

               v:static_text {
                  title = ")",
                  font = "<system/small>"
               }
            },
         },
      },
   }

end

--
-- Given a table of tags, ensure that items contain
-- no double quotes, then wrap items with double quotes.
--
function extra_tags.quote_all(tags)

   local quoted_tags = { }

   for i, item in ipairs(tags) do
      item = item:gsub('"+', '')      -- silently strip any doublequote that might remain

      if item:match('[^%s]') then
         table.insert(quoted_tags, '"' .. item .. '"')
      end
   end

   return quoted_tags
end

--
-- Given a table of tags, ensure that items contain
-- no commas
--
function extra_tags.no_commas(tags)

   local new_tags = { }

   for i, item in ipairs(tags) do
      item = item:gsub(',+', ' ')      -- silently strip any commas that might remain

      if item:match('[^%s]') then
         table.insert(new_tags, item)
      end
   end

   return new_tags
end

function extra_tags.as_table(state)

   local okay = true
   local tags = {}

   if not state[my.extra_tags_key] then
      return tags, okay
   end

   if state._extra_tags_style == 'space' then
      tags, okay = util.SsvToTable_NoEmpty(state[my.extra_tags_key])

      -- if a tag ends with a comma, it's probably a mistake, so flag it

     for i, item in pairs(tags) do
         if item:match(',$') then
            okay = false
            break
         end
      end
   else
      tags, okay = util.CsvToTable_NoEmpty(state[my.extra_tags_key])
   end

   return tags, okay

end


function extra_tags.exportPresetFields()

   return {
      { key = '_extra_tags_style', default = my.default },
   }

end


return function(args)
          my = args

          if not my.space_help then
             my.space_help = LOC("$$$/283=space-separated; enclose in quotes as needed, e.g.:")
          end
          if not my.space_example then
             my.space_example = LOC("$$$/284=flower  still-life  \"red rose\"")
          end
          if not my.comma_help then
             my.comma_help = LOC("$$$/223=comma-separated; enclose in quotes as needed, e.g.:")
          end
          if not my.comma_example then
             my.comma_example = LOC("$$$/224=flower,  with vase,  \"rose, red\"")
          end

          if not my.header then
             my.header = LOC("$$$/222=Extra tags")
          end
          if not my.tags then
             my.tags = LOC("$$$/433=tags")
          end
          if not my.tag then
             my.tag = LOC("$$$/434=tag")
          end

          return extra_tags
       end
