--
-- Copyright 2008 Jeffrey Friedl
-- http://regex.info/blog/
--
-- This module supports checking to see whether a new version of a plugin
-- is available
--

-- system imports
local LrBinding = import 'LrBinding'
local LrDialogs = import 'LrDialogs'
local LrColor   = import 'LrColor'
local LrDate    = import 'LrDate'
local LrView    = import 'LrView'
local LrHttp    = import 'LrHttp'
local LrMD5     = import 'LrMD5'

-- local imports
local async_task = require 'async_task'
local util       = require 'util'

local version_check = {}


local check_delta = 3600 * 12  -- how often to auto-check, in seconds


--
-- This file is loaded like
--
--    version_check = require 'version_check'('my name', MyVersion)
--
-- The 'require' returns a function that accepts the particulars of the
-- caller (its name and current version number), which is then immediately called.
-- That function fills in the 'my' table below, and returns a table that exposes the
-- public functions of this modules (currently, only 'do_async')
--
local my = {
   tag     = nil,
   version = nil,
   url     = nil,
}


--
-- Does a raw version check (that is, in the current task) and updates the
-- state appropriately. If 'force' is true, the check is in response to a
-- user having pressed a [Check Now] button. Otherwise, it's an auto-check
-- that we want to be light weight.
--
local function do_raw_check(state, force)

   --
   -- Contact my server to find out the latest version number. The user's
   -- current version and a "random unique identifier" (created by
   -- math.random) so that I can count, out of curiosity, how many
   -- individual users are using each plugin. The "random unique identifier"
   -- is used instead of an IP address because it's both more and less constant
   -- (more, because it doesn't change as a user's dynamic IP changes, and
   -- less because it differs among different users behind the same proxy).
   --

   local LrApplication = import 'LrApplication'
   local LrVersion
   if LrApplication.versionString then
      LrVersion = LrApplication.versionString()
   else
      LrVersion = "1.x"
   end

   local OS
   if WIN_ENV then
      OS = 'W'
   else
      OS = 'M'
   end

   local url2check = "http://regex.info/LightroomPlugins/v/" .. LrVersion .. "/" .. assert(my.tag)
                     .. "?current=" .. assert(my.version)
                     .. "&ruid="    .. state.version_check_ruid
                     .. "&os="      .. OS
                     .. "&LR="      .. LrVersion


   local reply = LrHttp.get(url2check)

   --
   -- Regardless of the reply, if this is an auto-check (as opposed to a
   -- user-initiated check), we want to mark that we did a check. This
   -- stops the thing from auto-checking over and over in the face of
   -- downed connectivity of a problem with the version server.
   --
   if not force then
      state.version_check_time = LrDate.currentTime()
   end

   if not reply or reply == '' then 
      -- Oh well, something went wrong. If this was a user-initiated check, let them known
      if force then
         LrDialogs.message(LOC("$$$/101=INTERNAL PLUGIN ERROR"),
                           LOC("$$$/102=Could not contact the version-check server"),
                           "warning")
      end
      return
   end

   -- got a reply... what kind?
   local LatestVersion = reply:match('^(%d+\.%d+)%s*$')

   if not LatestVersion then

      --
      -- We got no reply, so just leave things be unless its a forced
      -- check, in which case we report the problem
      --
      if force then
         LrDialogs.message(LOC("$$$/101=INTERNAL PLUGIN ERROR"),
                           LOC("$$$/103=Reply from the version server not understood:") .. "\n" .. tostring(reply),
                           "warning")
      end

      return
   end

   --
   -- We have a proper reply. We updated the check time earlier if it wasn't a forced check,
   -- but now we'll do so regardless.
   --
   state.version_check_time = LrDate.currentTime()


   -- We now actually check to see whether ours is old
   if tonumber(LatestVersion) > tonumber(my.version) then

      -- ours is old
      state.newer_version_number = LatestVersion
      state.newer_version_button_text = LOC("$$$/104=   Version ^1 Now Available   ", tostring(LatestVersion))
      state.info_synopsis = LOC("$$$/105=update available")

   else

      -- ours is not old
      state.newer_version_number = nil
      state.info_synopsis = ''
      if force then
         LrDialogs.message(LOC("$$$/106=You have the latest version"), nil, "info")
      end

   end
end

function version_check.do_async(state, force)
   if not state._checking_for_new_version and force or LrDate.currentTime() - state.version_check_time > check_delta then

      state._checking_for_new_version = true
      state._tmp_checknow_title = LOC("$$$/107=checking^.")

      async_task.run(state, LOC("$$$/385=checking for new version"),
                     do_raw_check, force,
                     function()
                        state._tmp_checknow_title = LOC("$$$/108=check now")
                        state._checking_for_new_version = false
                     end)
   end
end

function version_check.exportPresetFields()

   math.randomseed(os.time())
   local ruid = LrMD5.digest(tostring(os.clock()) .. tostring(math.random())) -- random unique identifier

   return {
      { key = 'auto_version_check',         default = true                                      },
      { key = 'version_check_time',         default = 0                                         },
      { key = 'newer_version_number',       default = nil                                       }, -- if not nil, a version that's newer than our own
      { key = 'newer_version_button_text',  default = '  Version XXXXXXXXX.X Now Available  '   }, -- fake value to reserve UI space
      { key = 'info_synopsis',              default = ''                                        },
      { key = 'version_check_ruid',         default = ruid                                      }, 
   }

end

--
-- Called while creating the service providers sectionsForBottomOfDialog, to provide
-- the "check for new versions?" UI.
--
function version_check.dialog_section(v, state)

   local function do_auto_version_check()
      if state.auto_version_check then
         version_check.do_async(state)
      end
   end

   state:addObserver('auto_version_check', do_auto_version_check)
   do_auto_version_check(state)

   state._checking_for_new_version = false
   state._tmp_checknow_title = LOC("$$$/108=check now")

   return v:view {
      fill_horizontal = 1,
      place = 'horizontal',
      margin_top = 13,
      margin_bottom = 13,
      
      v:checkbox {
         title = LOC("$$$/109=automatically check for updates"),
         value = LrView.bind('auto_version_check'),
      },

      v:push_button {
         title = LrView.bind('_tmp_checknow_title'),
         enabled = LrBinding.negativeOfKey('_checking_for_new_version'),
         action = function(button)
                     version_check.do_async(state, true)
                  end,
      },

      v:push_button {
         visible = LrBinding.keyIsNotNil('newer_version_number'),
         fill_horizontal = 1,
         font = "<system/bold>",
         title = LrView.bind('newer_version_button_text'),
         text_color = LrColor(1,0,0), -- Hmm, works on OSX only
         action = function(button)
                     LrHttp.openUrlInBrowser(my.url)
                  end,
      }
   }
end

function current_version()
   return my.version
end
function current_version_tag()
   return my.tag
end

return function(tag, version, url)
          my.tag = tag
          my.version = version
          my.url = url

          -- Return a table that exposes this module's public functions
          return version_check
       end
